#!/usr/bin/env python

import os

import gtk
from matplotlib.figure import Figure
from matplotlib import rc
from numpy import linspace, log, pi, power
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg \
                                            as FigureCanvas

import latexmath2png

rc('text', usetex=True)

class RadarEqApp(object):
    def __init__(self, title):
        eqs = [('eqn_1',
                      r'$R_{max}='
                      r'[\frac{P_tG^2\lambda^2\sigma}{(4\pi)^3S_{min}}]'
                      r'^{\frac{1}{4}}$'),
               ('eqn_2',
                      r'$R_{max}='
                      r'[\frac{P_tA_e^2\sigma}{4\pi\lambda^2S_{min}}]$'),
               ('pt',
                      r'$P_t$'),
               ('g',
                      r'$G$'),
               ('sigma',
                      r'$\sigma$'),
               ('smin',
                      r'$S_{min}$'),
               ('ae',
                      r'$A_e$'),
               ('watts',
                      r'$W$'),
               ('db',
                      r'$dB$'),
               ('m2',
                      r'$m^2$')]
        prefix = 'tex_'
        latexmath2png.math2png([latex for (eq, latex) in eqs],
                               os.getcwd(),
                               prefix=prefix,
                               size=1.25)

        self.__pngs = {}
        for (index, (eq, latex)) in enumerate(eqs):
            filename = '%s%d.png' % (prefix, index + 1)
            self.__pngs[eq] = filename

        self.__ui = {}

        self.__ui['window'] = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.__ui['window'].set_title(title)
        self.__ui['window'].set_size_request(640, 480)
        self.__ui['window'].set_border_width(10)
        self.__ui['window'].connect('destroy', self.__on_window_destroy)

        self.__ui['notebook'] = gtk.Notebook()
        self.__ui['notebook'].set_tab_pos(gtk.POS_TOP)
        self.__ui['window'].add(self.__ui['notebook'])

        self.__ui['tables'] = [gtk.Table(5, 4), gtk.Table(5, 4)]

        tabimage = gtk.image_new_from_file(self.__pngs['eqn_1'])
        self.__ui['notebook'].append_page(self.__ui['tables'][0], tabimage)
        tabimage = gtk.image_new_from_file(self.__pngs['eqn_2'])
        self.__ui['notebook'].append_page(self.__ui['tables'][1], tabimage)

        self.__ui['spinbuttons'] = [{}, {}]
        fields = [('pt', 'watts'),
                  ('g', 'db'),
                  ('sigma', 'm2'),
                  ('smin', 'watts')]
        self.__pack_fields(fields, page=0)
        fields = [('pt', 'watts'),
                  ('ae', 'm2'),
                  ('sigma', 'm2'),
                  ('smin', 'watts')]
        self.__pack_fields(fields, page=1)

        self.__figures = [Figure(figsize=(5, 4), dpi=100),
                          Figure(figsize=(5, 4), dpi=100)]
        self.__plots = [None, None]
        style = self.__ui['window'].rc_get_style()
        color = str(style.bg[gtk.STATE_NORMAL])

        self.__plots = [f.add_subplot(1, 1, 1) for f in self.__figures]
        for (figure, plot) in zip(self.__figures, self.__plots):
            figure.patch.set(facecolor=color)
            plot.hold(False)

        self.__ui['canvases'] = [FigureCanvas(f) for f in self.__figures]
        for (page, canvas) in enumerate(self.__ui['canvases']):
            self.__ui['tables'][page].attach(canvas, 3, 4, 0, 5)

        self.__plot(0)
        self.__plot(1)
        self.__ui['window'].show_all()

    def __plot(self, page):
        wave = linspace(0.01, 0.1, num=100)
        values = {}
        for field in ['pt', 'g', 'ae', 'sigma', 'smin']:
            spinbutton = self.__ui['spinbuttons'][page].get(field)
            if spinbutton:
                values[field] = spinbutton.get_value()
        styles = ['b-', 'r-']

        if page == 0:
            values['g'] = 10 ** (values['g'] / 10)
            rmax = power(((values['pt']) *
                          (values['g'] ** 2) *
                          (wave ** 2) *
                          (values['sigma']))/
                         (((4 * pi) ** 3) *
                          (values['smin'])),
                         0.25)
        elif page == 1:
            rmax = (((values['pt']) *
                     (values['ae'] ** 2) *
                     (values['sigma'])
                    )/
                    ((4 * pi) *
                     (wave ** 2) *
                     (values['smin'])))

        self.__plots[page].plot(wave, rmax, styles[page])
        self.__plots[page].set_xlabel(r'$\lambda$ (m)')
        self.__plots[page].set_ylabel(r'$R_{max}$ (m)')
        self.__plots[page].grid(True)
        self.__ui['canvases'][page].draw()

    def __pack_fields(self, fields, page):
        spindata = {'pt' : ((100000, 10000, 1000000, 10000, 100000, 0), 2),
                 'g' : ((50, 1, 100, 1, 10, 0), 2),
                 'ae' : ((0.01, 0.001, 0.1, 0.001, 0.01, 0), 4),
                 'sigma' : ((0.01, 0.001, 0.1, 0.001, 0.01, 0), 4),
                 'smin' : ((0.1, 0.1, 1, 0.1, 0.3, 0), 2)}

        for (index, (label, units)) in enumerate(fields):
            image = gtk.image_new_from_file(self.__pngs[label])
            self.__ui['tables'][page].attach(image,
                                          0,
                                          1,
                                          index,
                                          index + 1,
                                          xoptions=0,
                                          yoptions=0)

            spinbutton = gtk.SpinButton(gtk.Adjustment(*spindata[label][0]),
                                        0.5,
                                        spindata[label][1])
            spinbutton.connect('value-changed', lambda w: self.__plot(page))
            self.__ui['tables'][page].attach(spinbutton,
                                          1,
                                          2,
                                          index,
                                          index + 1,
                                          xoptions=0,
                                          yoptions=0)
            self.__ui['spinbuttons'][page][label] = spinbutton

            image = gtk.image_new_from_file(self.__pngs[units])
            self.__ui['tables'][page].attach(image,
                                          2,
                                          3,
                                          index,
                                          index + 1,
                                          xoptions=0,
                                          yoptions=0)

    def __on_window_destroy(self, widget):
        for png in self.__pngs.values():
            os.unlink(os.path.join(os.getcwd(), png))

        gtk.main_quit()

    def main(self):
        gtk.main()

if __name__ == '__main__':
    app = RadarEqApp('Radar Equation Plotter')
    app.main()

